package fr.labri.collections;

import java.util.List;

public interface ITree {
	public final String NO_LABEL = "";
	
	/**
	 * Add the given tree as a child, and update its parent.
	 * @param t
	 */
	public abstract void addChild(ITree t);

	/**
	 * Make a deep copy of the tree.
	 * @return a deep copy of the tree.
	 */
	public abstract ITree deepCopy();

	/**
	 * Returns the position of the given child in the tree. 
	 * @param child
	 * @return the position of the child, or -1 if the given child is not in the children list.
	 */
	public abstract int getChildPosition(ITree child);

	public abstract List<ITree> getChildren();
	public abstract ITree getChild(int position);

	/**
	 * Returns all the descendants of the tree, using a pre-order.
	 * @return
	 */
	public abstract List<ITree> getDescendants();

	public abstract boolean hasLabel();
	public abstract String getLabel();

	public abstract List<ITree> getLeaves();

	public abstract ITree getParent();

	/**
	 * Returns all the parents of the tree.
	 * @return
	 */
	public abstract List<ITree> getParents();

	/**
	 * Return all the nodes contains in the tree, using a pre-order.
	 * @return
	 */
	public abstract List<ITree> getTrees();

	/**
	 * Returns if this tree is isomorphic to another the other
	 * @param other another tree
	 * @return true if both tree are isomorphics
	 */
	public abstract boolean isIsomorphicTo(ITree other);
	
	public abstract int getType();
	public abstract void setType(int type);

	public abstract int hashCode();

	/**
	 * Indicate if the trees have the same type.
	 * @param t
	 * @return
	 */
	public abstract boolean isCompatible(ITree t);

	/**
	 * Indicate whether or not the tree has children.
	 * @return
	 */
	public abstract boolean isLeaf();

	public abstract boolean isRoot();

	/**
	 * Indicate whether or not the tree is similar to the given tree.
	 * @param t
	 * @return true if they are compatible and have same label, false either
	 */
	public abstract boolean isSimilar(ITree t);

	public abstract Iterable<ITree> postOrder();

	public abstract Iterable<ITree> breadthFirst();

	public abstract int positionInParent();

	public abstract void setChildren(List<ITree> children);

	public abstract void setLabel(String label);

	public abstract void setParent(ITree parent);
	public abstract void setParentAndUpdateChildren(ITree parent);

}
