package fr.labri.collections;

import java.util.ArrayList;
import java.util.List;

public class Tree extends AbstractTree implements ITree {
	
	// Type of the token
	int type;

	// Label of the token
	String label;
	
	public Tree(int type, String label) {
		this.type = type;
		this.label = (label == null) ? NO_LABEL : label.intern();
		this.children = new ArrayList<>();
	}
	
	// Only used for cloning ...
	private Tree(Tree other) {
		this.type = other.getType();
		this.label = other.getLabel();
		
		this.children = new ArrayList<>();
	}
	
	@Override
	public void addChild(ITree t) {
		children.add(t);
		t.setParent(this);
	}

	@Override
	public Tree deepCopy() {
		Tree copy = new Tree(this);
		for (ITree child: getChildren())
			copy.addChild(child.deepCopy());
		return copy;
	}

	@Override
	public List<ITree> getChildren() {
		return children;
	}

	@Override
	public String getLabel() {
		return label;
	}

	@Override
	public ITree getParent() {
		return parent;
	}

	@Override
	public int getType() {
		return type;
	}
	
	@Override
	public void setChildren(List<ITree> children) {
		this.children = children;
		for (ITree c: children)
			c.setParent(this);
	}

	@Override
	public void setLabel(String label) {
		this.label = label;
	}

	@Override
	public void setParent(ITree parent) {
		this.parent = parent;
	}

	@Override
	public void setParentAndUpdateChildren(ITree parent) {
		if (this.parent != null) this.parent.getChildren().remove(this);
		this.parent = parent;
		if (this.parent != null) parent.getChildren().add(this);
	}

	@Override
	public void setType(int type) {
		this.type = type;
	}
}
